(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "./util", "./expr", "./window"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("./util"), require("./expr"), require("./window"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.util, global.expr, global.window);
    global.over = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _util, _expr, _window) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.overToSQL = overToSQL;

  function overToSQL(over) {
    if (!over) return;
    const {
      as_window_specification: asWindowSpec,
      expr,
      keyword,
      orderby,
      partitionby,
      type
    } = over;

    if ((0, _util.toUpper)(type) === 'WINDOW') {
      const windowSQL = (0, _window.asWindowSpecToSQL)(asWindowSpec);
      return `OVER ${windowSQL}`;
    }

    if ((0, _util.toUpper)(type) === 'ON UPDATE') {
      let onUpdate = `${(0, _util.toUpper)(type)} ${(0, _util.toUpper)(keyword)}`;
      const args = (0, _expr.exprToSQL)(expr);
      if (args) onUpdate = `${onUpdate}(${args.join(', ')})`;
      return onUpdate;
    }

    const partition = (0, _expr.orderOrPartitionByToSQL)(partitionby, 'partition by');
    const order = (0, _expr.orderOrPartitionByToSQL)(orderby, 'order by');
    return `OVER (${[partition, order].filter(_util.hasVal).join(' ')})`;
  }
});